import { SimplifyJSCase, SimplifyJSAltCase } from './simplifyJS.js'
import {
  TransformToArrayFormCase,
  TransformToObjectFormCase,
  OrigSimplifyWithTransformCase
} from './transforms.js'
import {
  SimplifyWASMCase,
  StoreCoordsCase,
  LoadResultCase
} from './simplifyWasm.js'

export const chartDependencies = {
  wasmStack: [StoreCoordsCase, SimplifyWASMCase, LoadResultCase],
  simplifyJSStack: [
    TransformToObjectFormCase,
    OrigSimplifyWithTransformCase,
    TransformToArrayFormCase
  ],
  wasmVsJs: [SimplifyWASMCase, SimplifyJSCase, SimplifyJSAltCase]
}

const uniqueFilter = (val, idx, self) => self.indexOf(val) === idx

export function getChartDependencies(chartNames) {
  return chartNames
    .map(name => chartDependencies[name])
    .flat()
    .filter(uniqueFilter)
}

export function generateCases(data, { tolRange, highQual, chart }) {
  if (!tolRange) return []

  const cases = []
  let params = { data, highQuality: highQual }
  for (let tol of tolRange) {
    params = { ...params, tol }
    for (let BenchCaseClass of chartDependencies[chart]) {
      cases.push(new BenchCaseClass(params))
    }
  }
  return cases
}
