import simplifyJS from '../../../lib/simplify-js/simplify.js'
import { Case } from './Case.js'

function arrayToObjectFormat(coords) {
  return coords.map(coord => {
    return { x: coord[0], y: coord[1] }
  })
}

function objectToArrayFormat(coords) {
  return coords.map(coord => [coord.x, coord.y])
}

export class TransformToObjectFormCase extends Case {
  get name() {
    return 'transformToObjectFormCase'
  }
  async fn() {
    arrayToObjectFormat(this.parameters.data)
  }
}

export class TransformToArrayFormCase extends Case {
  get name() {
    return 'transformToArrayFormCase'
  }
  async setup() {
    const { data, tol, highQuality } = this.parameters
    const transformed = arrayToObjectFormat(data)
    this.parameters.simplified = simplifyJS(transformed, tol, highQuality)
  }
  async fn() {
    objectToArrayFormat(this.parameters.simplified)
  }
}

export class OrigSimplifyWithTransformCase extends Case {
  get name() {
    return 'origSimplifyWithTransformCase'
  }
  async fn() {
    const { data, tol, highQuality } = this.parameters
    const transformed = arrayToObjectFormat(data)
    const simplified = simplifyJS(transformed, tol, highQuality)
    objectToArrayFormat(simplified)
  }
}
