function genDataset(id, label, color, yAxisID = 'performance', type) {
  return {
    id,
    type,
    label,
    yAxisID,
    fill: false,
    backgroundColor: color,
    borderColor: color,
    data: [],
    stack: 'a'
  }
}

export const datasetTemplates = {
  numberOfNodes: genDataset(
    'numberOfNodes',
    '# of positions',
    'grey',
    'nodes',
    'line'
  ),
  simplifyWASM: genDataset('simplifyWASM', 'Simplify.wasm', 'red'),
  simplifyJS: genDataset('simplifyJS', 'Simplify.js', 'blue'),
  simplifyJSAlt: genDataset(
    'simplifyJSAlt',
    'Simplify.js Alternative',
    'green'
  ),
  storeCoords: genDataset('storeCoords', 'Store coordinates', 'blue'),
  loadResult: genDataset('loadResult', 'Load result', 'green'),
  origSimplifyWithTransformCase: genDataset(
    'origSimplifyWithTransformCase',
    'Simplify.js with data transformation',
    'red'
  ),
  transformToArrayFormCase: genDataset(
    'transformToArrayFormCase',
    'Transform result to array format',
    'green'
  ),
  transformToObjectFormCase: genDataset(
    'transformToObjectFormCase',
    'Transform coordinates to object format',
    'blue'
  )
}

function genScaleLabel(label) {
  return {
    display: true,
    labelString: label,
    fontStyle: 'bold'
  }
}

export function genOptions(
  title,
  stacked = false,
  scaleY = 'linear',
  perfUnit = 'mean'
) {
  const perfUnitText = {
    mean: 'Milliseconds per operation (ms)',
    hz: 'Operations per second (hz)'
  }
  return {
    animation: {
      duration: 0
    },
    legend: {
      position: 'bottom'
    },
    title: {
      display: true,
      text: title
    },
    tooltips: {
      mode: 'index',
      intersect: false
    },
    hover: {
      mode: 'nearest',
      intersect: false
    },
    scales: {
      xAxes: [
        {
          scaleLabel: genScaleLabel('Tolerance (length)'),
          stacked
        }
      ],
      yAxes: [
        {
          id: 'performance',
          type: scaleY,
          stacked,
          ticks: {
            min: 0
          },
          scaleLabel: genScaleLabel(`Performance - ${perfUnitText[perfUnit]}`)
        },
        {
          id: 'nodes',
          position: 'right',
          type: scaleY,
          gridLines: {
            drawOnChartArea: false
          },
          ticks: {
            min: 0
          },
          scaleLabel: genScaleLabel('Number of positions in result (#)')
        }
      ]
    }
  }
}

/**
 * Show a maximum precision
 * @param {array} labels
 */
export function beautifyLabels(labels, precision = 5) {
  let prec = Math.pow(10, precision)
  return labels.map(label => {
    let rounded = Math.round(label * prec) / prec
    return rounded === 0 ? label : rounded
  })
}
