import React, { Component } from 'react'

import { datasetNames } from '../data/index.js'

export class Form extends Component {
  constructor(props) {
    super(props)
    this.state = {
      tolStart: 0.1,
      tolStep: 0.1,
      tolEnd: 1,
      highQual: true,
      chart: 'wasmVsJs',
      benchMethod: 'iteration',
      iterations: 10,
      timeToRun: 200,
      dataset: Object.keys(datasetNames)[0],
      debouncedChart: false
    }

    this.handleInputChange = this.handleInputChange.bind(this)
  }

  componentDidUpdate(prevProps, prevState) {
    if (prevState !== this.state) {
      this.notifyParent()
    }
  }

  componentDidMount() {
    this.notifyParent()
  }

  handleInputChange(event) {
    let name, value
    let target = event.target
    name = target.name
    value = target.type === 'checkbox' ? target.checked : target.value
    value = target.type === 'number' ? Number(value) : value
    value = target.type === 'range' ? Number(value) : value
    this.setState({ [name]: value })
  }

  async notifyParent() {
    let { tolStart, tolEnd, tolStep } = this.state
    let tolRange = calculateRange(tolStart, tolEnd, tolStep)
    this.props.onFormChange({ ...this.state, tolRange })
  }

  /** changes order of magnitude for all tolerance values */
  changeOOM(mode) {
    let maxPrecision = 6
    let p = Math.pow(10, maxPrecision)
    let mod = mode === 'increase' ? 10 : 0.1
    let { tolStart, tolStep, tolEnd } = this.state
    const round = val => Math.round(val * p) / p
    const modify = val => (round(val * mod) === 0 ? val : round(val * mod))
    this.setState({
      tolStart: modify(tolStart),
      tolStep: modify(tolStep),
      tolEnd: modify(tolEnd)
    })
  }

  render() {
    return (
      <div className="component">
        <h2>Settings</h2>
        <div className="form">
          <label>Dataset</label>
          <select
            name="dataset"
            value={this.state.dataset}
            onChange={this.handleInputChange}
          >
            {Object.keys(datasetNames).map((key, idx) => (
              <option value={key} key={idx}>
                {datasetNames[key]}
              </option>
            ))}
          </select>

          <label>Tolerance order of magnitude</label>
          <div>
            <button onClick={() => this.changeOOM('increase')}>+</button>
            <button onClick={() => this.changeOOM('decrease')}>-</button>
          </div>

          <label>Tolerance start: </label>
          <input
            name="tolStart"
            type="number"
            min="0.0001"
            max="1"
            step="0.0001"
            value={this.state.tolStart}
            onChange={this.handleInputChange}
          />

          <label>Tolerance step: </label>
          <input
            name="tolStep"
            type="number"
            min="0.0001"
            max="1"
            step="0.0001"
            value={this.state.tolStep}
            onChange={this.handleInputChange}
          />

          <label>Tolerance end: </label>
          <input
            name="tolEnd"
            type="number"
            min="0.001"
            max="5"
            step="0.001"
            value={this.state.tolEnd}
            onChange={this.handleInputChange}
          />

          <label>high Quality: </label>
          <input
            name="highQual"
            type="checkbox"
            checked={this.state.highQual}
            onChange={this.handleInputChange}
          />

          <label>Chart: </label>
          <select
            name="chart"
            value={this.state.chart}
            onChange={this.handleInputChange}
          >
            <option value="wasmVsJs">simplifyWasm vs. simplifyJS</option>
            <option value="wasmStack">wasmStack</option>
            <option value="simplifyJSStack">simpslifyJSStack</option>
          </select>

          <label>Benchmarking method: </label>
          <select
            name="benchMethod"
            value={this.state.benchMethod}
            onChange={this.handleInputChange}
          >
            <option value="iteration">
              Iterations (fast - low significance)
            </option>
            <option value="benchmarkJS">
              BenchmarkJS (slow - statistically significant)
            </option>
            <option value="opsPerTime">
              OpsPerTime (fixed time - low significance)
            </option>
          </select>
          {this.state.benchMethod === 'iteration' && (
            <label>Iterations ({this.state.iterations})</label>
          )}
          {this.state.benchMethod === 'iteration' && (
            <input
              name="iterations"
              type="range"
              min="1"
              max="100"
              step="1"
              value={this.state.iterations}
              onChange={this.handleInputChange}
            ></input>
          )}
          {this.state.benchMethod === 'opsPerTime' && (
            <label>Time to run ({this.state.timeToRun}ms)</label>
          )}
          {this.state.benchMethod === 'opsPerTime' && (
            <input
              name="timeToRun"
              type="range"
              min="100"
              max="1000"
              step="10"
              value={this.state.timeToRun}
              onChange={this.handleInputChange}
            ></input>
          )}

          <label>Debounce Chart rendering: </label>
          <input
            name="debouncedChart"
            type="checkbox"
            checked={this.state.debouncedChart}
            onChange={this.handleInputChange}
          />
        </div>
      </div>
    )
  }
}

function calculateRange(start, stop, step) {
  const range = []
  for (start; start < stop; start += step) {
    range.push(start)
    if (step === 0) break
    if (range.length > 200) break
  }
  return range
}
