import React, { Component } from 'react'

import { dataSelector } from '../data/index.js'

import { BenchmarkSuite } from '../benchmarks/BenchmarkSuite.js'
import { generateCases } from '../benchmarkCases/index.js'
import { benchmarkTypeSelector } from '../benchmarks/index.js'

import { Progress } from './progress'

export class BenchmarkRunner extends Component {
  constructor(props) {
    super(props)

    this.state = {
      state: 'initialized',
      totalCount: 0,
      finishedCount: 0
    }

    this.suite = new BenchmarkSuite()

    this.suite.onChange = () => {
      this.setState({
        state: this.suite.state,
        totalCount: this.suite.cases.cases.length,
        finishedCount: this.suite.cases.idx + 1
      })
    }
  }

  componentDidMount() {
    this.props.onStatsChange(this.suite.stats)
  }

  async componentDidUpdate(prevProps, prevState) {
    if (prevState !== this.state) {
      // state changed -> internal update
      this.props.onStatsChange(this.suite.stats)
    }
    if (prevProps.formState !== this.props.formState) {
      // props changed -> update from parent
      await this.initializeSuite()
      this.suite.stop()
    }
  }

  async initializeSuite() {
    let data = await dataSelector(this.props.formState.dataset)
    this.suite.setBenchmarkType(benchmarkTypeSelector(this.props.formState))
    this.suite.setCases(generateCases(data, this.props.formState))
  }

  stateIsOneOf(states) {
    return states.indexOf(this.suite.state) !== -1
  }

  stopButtonEnabled() {
    return this.stateIsOneOf(['running', 'finished', 'paused'])
  }

  startButtonShown() {
    return this.stateIsOneOf(['initialized'])
  }

  resumeButtonShown() {
    return this.stateIsOneOf(['paused'])
  }

  pauseButtonShown() {
    return this.stateIsOneOf(['running'])
  }

  render() {
    return (
      <div className="component">
        <Progress
          totalCount={this.state.totalCount}
          finishedCount={this.state.finishedCount}
        />
        <div>State: {this.suite.state}</div>
        <div>
          <button
            onClick={() => this.suite.stop()}
            disabled={!this.stopButtonEnabled()}
          >
            stop
          </button>

          {this.startButtonShown() && (
            <button
              onClick={async () => {
                await this.initializeSuite()
                this.suite.start()
              }}
            >
              start
            </button>
          )}

          {this.resumeButtonShown() && (
            <button onClick={() => this.suite.start()}>start</button>
          )}

          {this.pauseButtonShown() && (
            <button onClick={() => this.suite.pause()}>pause</button>
          )}
        </div>
      </div>
    )
  }
}
